@@ -1,4 +1,4 @@
-# Copyright 2018 Venelin Mitov
+# Copyright 2016-2019 Venelin Mitov
 #
 # This file is part of PCMBase.
 #
@@ -54,7 +54,7 @@ PCMCond.White <- function(
   metaI = PCMInfo(NULL, tree, model, verbose = verbose),
   verbose=FALSE) {
   if(!is.null(model$Sigmae_x)) {
-    Sigmae_x <- if(is.Global(model$Sigmae_x)) as.matrix(model$Sigmae_x) else as.matrix(model$Sigmae_x[,,r])
+    Sigmae_x <- GetSigma_x(model, "Sigmae", r)
     Sigmae <- Sigmae_x %*% t(Sigmae_x)
   } else {
     Sigmae <- NULL
@@ -81,7 +81,7 @@ PCMCond.White <- function(
 PCMDescribeParameters.White <- function(model, ...) {
   list(
     X0 = "trait values at the root of the tree (for White model this is the mean vector)",
-    Sigmae_x = "Choleski factor of the non-phylogenetic variance-covariance matrix")
+    Sigmae_x = "factor of the non-phylogenetic variance-covariance matrix")
 }
 
 #' @export
@@ -106,7 +106,7 @@ PCMSpecify.White <- function(model, ...) {
     X0 = structure(0.0, class = c('VectorParameter', '_Global'),
                    description = 'trait values at the root of the tree (for White model this is the mean vector)'),
     Sigmae_x = structure(0.0, class = c('MatrixParameter', '_UpperTriangularWithDiagonal', '_WithNonNegativeDiagonal'),
-                         description = 'Choleski factor of the non-phylogenetic variance-covariance matrix'))
+                         description = 'factor of the non-phylogenetic variance-covariance matrix'))
   attributes(spec) <- attributes(model)
   if(is.null(names(spec))) names(spec) <- c('X0', 'Sigmae_x')
   if(any(sapply(spec, is.Transformable))) class(spec) <- c(class(spec), '_Transformable')
